import dataiku
from dataiku.customrecipe import *
import pandas as pd, numpy as np
from dataiku import pandasutils as pdu
from lifelines import KaplanMeierFitter
from matplotlib import pyplot as plt
from pandas.api.types import is_integer_dtype, is_bool_dtype


event_dataset = dataiku.Dataset(get_input_names_for_role('event_dataset')[0])
survival_dataset = dataiku.Dataset(get_output_names_for_role('survival_dataset')[0])

plots_folder = get_output_names_for_role('plots_folder')
if len(plots_folder) > 0:
    plots_folder = dataiku.Folder(get_output_names_for_role('plots_folder')[0])
else:
    plots_folder = None

event_df = event_dataset.get_dataframe()

duration = get_recipe_config()['duration']
if not is_integer_dtype(event_df[duration]):
    raise ValueError('Duration column {} should be of type int but is of type {}'.format(duration, event_df[duration].dtype))

death = get_recipe_config()['death']
if not is_bool_dtype(event_df[death]):
    raise ValueError('Death column {} should be of type bool but is of type {}'.format(death, event_df[death].dtype))

condition = get_recipe_config()['condition']
custom_dates = get_recipe_config()['custom_dates']
start = int(get_recipe_config()['start'])
step = int(get_recipe_config()['step'])
stop = int(get_recipe_config()['stop'])

survival = []
timeline = None

for c in event_df[condition].unique():
    cond_df = event_df[event_df[condition] == c]
    if cond_df.shape[0] == 0:
        continue

    T = cond_df[duration]
    E = cond_df[death]

    kmf = KaplanMeierFitter()
    kmf.fit(T, event_observed=E)
    
    if not custom_dates:
        res = pd.concat([kmf.survival_function_, kmf.confidence_interval_], axis=1).reset_index()
        res['condition'] = c
        survival.append(res)
        print(survival[-1])
        if plots_folder is not None:
            kmf.plot()
            plt.savefig(os.path.join(plots_folder.get_path(), str(c) + '.png'))
            plt.close()
    else:
        res = kmf.predict(list(range(start, stop, step)) + [stop])
        res = pd.DataFrame({c: res})
        survival.append(res)
        if plots_folder is not None:
            kmf.plot(loc=slice(start, stop))
            plt.savefig(os.path.join(plots_folder.get_path(), str(c) + '.png'))
            plt.close()

if not custom_dates:
    survival_dataset.write_with_schema(pd.concat(survival, axis=0))
else:
    res = pd.concat(survival, axis=1)
    res['timeline'] = timeline
    survival_dataset.write_with_schema(res.reset_index())